@@ -41,7 +41,7 @@ namespace FSpot {
 		
 		[Glade.Widget] Gtk.RadioButton static_radio;
 		[Glade.Widget] Gtk.RadioButton original_radio;
-		[Glade.Widget] Gtk.RadioButton none_radio;
+		[Glade.Widget] Gtk.RadioButton plain_radio;
 
 		[Glade.Widget] Gtk.SpinButton size_spin;
 
@@ -62,6 +62,7 @@ namespace FSpot {
 		
 		public FolderExport (IPhotoCollection selection) : base ("folder_export_dialog")
 		{
+			/*
 			Gnome.Vfs.ModuleCallbackFullAuthentication auth = new Gnome.Vfs.ModuleCallbackFullAuthentication ();
 			auth.Callback += new Gnome.Vfs.ModuleCallbackHandler (HandleAuth);
 			auth.SetDefault ();
@@ -81,7 +82,7 @@ namespace FSpot {
 			msg.Callback += new Gnome.Vfs.ModuleCallbackHandler (HandleMsg);
 			msg.SetDefault ();
 			msg.Push ();
-			
+			*/
 			this.selection = selection;
 			
 			IconView view = (IconView) new IconView (selection);
@@ -94,7 +95,7 @@ namespace FSpot {
 			thumb_scrolledwindow.Add (view);
 			HandleSizeActive (null, null);
 			name_entry.Text = gallery_name;
-			uri_entry.Text = System.IO.Path.Combine (FSpot.Global.HomeDirectory, "public_html");
+			uri_entry.Text = System.IO.Path.Combine (FSpot.Global.HomeDirectory, "Desktop");
 			if (!System.IO.Directory.Exists (uri_entry.Text))
 				uri_entry.Text = FSpot.Global.HomeDirectory;
 
@@ -116,7 +117,7 @@ namespace FSpot {
 			Gnome.Vfs.Result result = Gnome.Vfs.Result.Ok;
 
 			try {
-				Dialog.Destroy ();
+				Dialog.Hide ();
 				
 				Gnome.Vfs.Uri source = new Gnome.Vfs.Uri (Path.Combine (gallery_path, gallery_name));
 				Gnome.Vfs.Uri target = dest.Clone();
@@ -137,9 +138,13 @@ namespace FSpot {
 					gallery = new FolderGallery (selection, gallery_path, gallery_name);
 				}
 
-				if (scale)
+				if (scale) {
+					System.Console.WriteLine ("setting scale to {0}", size);
 					gallery.SetScale (size);
-				
+				} else {
+					System.Console.WriteLine ("Exporting full size image");
+				}
+
 				gallery.Description = description;
 
 				gallery.Generate ();
@@ -170,7 +175,7 @@ namespace FSpot {
 				}
 
 				if (open)
-					Gnome.Url.Show (target.ToString ());
+					GnomeUtil.UrlShow (null, target.ToString ());
 
 			} catch (System.Exception e) {
 				progress_dialog.Message = e.ToString ();
@@ -180,6 +185,7 @@ namespace FSpot {
 				// created.
 				if (!dest.IsLocal)
 					System.IO.Directory.Delete (gallery_path, true);
+				Dialog.Destroy ();
 			}
 		}
 		
@@ -193,10 +199,10 @@ namespace FSpot {
 			
 			switch (info.Status) {
 			case Gnome.Vfs.XferProgressStatus.Vfserror:
-				progress_dialog.Message = Mono.Posix.Catalog.GetString ("Error: Error while transferring, Aborting");
+				progress_dialog.Message = Mono.Posix.Catalog.GetString ("Error: Error while transferring; Aborting");
 				return (int)Gnome.Vfs.XferErrorAction.Abort;
 			case Gnome.Vfs.XferProgressStatus.Overwrite:
-				progress_dialog.ProgressText = Mono.Posix.Catalog.GetString ("Error: File Already Exists, Aborting");
+				progress_dialog.ProgressText = Mono.Posix.Catalog.GetString ("Error: File Already Exists; Aborting");
 				return (int)Gnome.Vfs.XferOverwriteAction.Abort;
 			default:
 				return 1;
@@ -301,7 +307,7 @@ namespace FSpot {
 		{
 			MakeDir (gallery_path);
 
-			for (int i = 0; i < collection.Photos.Length; i++)
+			for (int i = 0; i < collection.Count; i++)
 			{
 				ProcessImage (i);
 			}
@@ -309,12 +315,12 @@ namespace FSpot {
 		
 		protected virtual string ImageName (int image_num)
 		{
-			return System.IO.Path.GetFileName (collection.Photos [image_num].DefaultVersionPath); 
+			return System.IO.Path.GetFileName (collection [image_num].DefaultVersionUri.LocalPath); 
 		}
 
 		public void ProcessImage (int image_num)
 		{
-			Photo photo = collection.Photos [image_num];
+			Photo photo = (Photo) collection [image_num];
 			string photo_path = photo.DefaultVersionPath;
 			string path;
 			ScaleRequest req;
@@ -398,11 +404,13 @@ namespace FSpot {
 				return destination;
 			}
 			set {
-				this.destination = destination;
+				this.destination = value;
 			}
 		}
 
 		public void SetScale (int size) {
+			this.scale = true;
+			this.size = size;
 			requests [0].Width = size;
 			requests [0].Height = size;
 		}
@@ -430,24 +438,21 @@ namespace FSpot {
 		
 		protected override string ImageName (int photo_index)
 		{
-			return String.Format ("img-{0}.jpg", photo_index);
+			return String.Format ("img-{0}.jpg", photo_index + 1);
 		}
 
-		private string AlternateName (int photo_index, string extention) 
+		private string AlternateName (int photo_index, string extension) 
 		{
-			return System.IO.Path.GetFileNameWithoutExtension (ImageName (photo_index)) + extention;
+			return System.IO.Path.GetFileNameWithoutExtension (ImageName (photo_index)) + extension;
 		}
 
 		private void SetTime ()
 		{
 			try {
-				int i = 0;
-				foreach (Photo photo in collection.Photos) {
-					CreateComments (photo.DefaultVersionPath, i);
-					i++;
-				}
+				for (int i = 0; i < collection.Count; i++)
+					CreateComments (collection [i].DefaultVersionUri.LocalPath, i);
 
-				Directory.SetLastWriteTimeUtc(gallery_path, collection.Photos [0].Time);
+				Directory.SetLastWriteTimeUtc(gallery_path, collection [0].Time);
 
 				if (System.IO.Directory.Exists (SubdirPath ("mq")))
 				    CreateZipFile("mq");
@@ -462,8 +467,9 @@ namespace FSpot {
 
 		private void CreateComments(string photo_path, int photo_index)
 		{
-			StreamWriter comment = File.CreateText(SubdirPath  ("comments", "img-" + photo_index + ".txt"));
-			comment.WriteLine("<span>image " + photo_index + "</span>\n");
+			StreamWriter comment = File.CreateText(SubdirPath  ("comments", AlternateName (photo_index, "txt")));
+			comment.Write("<span>image " + photo_index + "</span> ");
+			comment.Write (collection [photo_index].Description + "\n");
 			comment.Close();
 		}
 
@@ -517,7 +523,9 @@ namespace FSpot {
 		private void CreateInfo()
 		{
 			StreamWriter info = File.CreateText(Path.Combine (gallery_path, "info.txt"));
-			info.WriteLine("date|" + collection.Photos[0].Time.Date.ToString ("dd.MM.yyyy"));
+			info.WriteLine("name|" + gallery_name);
+			info.WriteLine("date|" + collection [0].Time.Date.ToString ("dd.MM.yyyy"));
+			info.WriteLine("description|" + description);
 			info.Close();
 		}
 	}
@@ -526,7 +534,9 @@ namespace FSpot {
 	{
 		int current;
 		int perpage = 16;
-		string stylesheet = "f-spot-simple-white.css";
+		string stylesheet = "f-spot-simple.css";
+		string altstylesheet = "f-spot-simple-white.css";
+		string javascript = "f-spot.js";
 		
 		public HtmlGallery (IPhotoCollection selection, string path, string name) : base (selection, path, name) 
 		{ 
@@ -537,12 +547,12 @@ namespace FSpot {
 		
 		public override void Generate ()
 		{
-			if (collection.Photos.Length == 0)
+			if (collection.Count == 0)
 				return;
 			
 			base.Generate ();
 			
-			Photo [] photos = collection.Photos;
+			Photo [] photos = (Photo []) collection.Items;
 			
 			int i;
 			for (i = 0; i < photos.Length; i++)
@@ -562,6 +572,30 @@ namespace FSpot {
 				fs.Write (buffer, 0,  n);						    
 			
 			fs.Close ();
+			/* quick and stupid solution
+			   this should have been iterated over an array of stylesheets, really
+			*/
+			s = assembly.GetManifestResourceStream (altstylesheet);
+			fs = System.IO.File.Open (SubdirPath ("style", altstylesheet), System.IO.FileMode.Create);
+
+			buffer = new byte [8192];
+			n = 0;
+			while ((n = s.Read (buffer, 0, buffer.Length)) != 0)
+				fs.Write (buffer, 0,  n);						    
+			
+			fs.Close ();
+			/* Javascript for persistant style change */
+			MakeDir (SubdirPath ("script"));
+		  s = assembly.GetManifestResourceStream (javascript);
+			fs = System.IO.File.Open (SubdirPath ("script", javascript), System.IO.FileMode.Create);
+
+			buffer = new byte [8192];
+			n = 0;
+			while ((n = s.Read (buffer, 0, buffer.Length)) != 0)
+				fs.Write (buffer, 0,  n);						    
+			
+			fs.Close ();
+	
 		}
 		
 		public int PageCount {
@@ -611,29 +645,39 @@ namespace FSpot {
 			//writer.Indent = 4;
 			
 			//writer.Write ("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
+			writer.WriteLine ("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">");
 			writer.RenderBeginTag ("html");
 			
 			WriteHeader (writer);
 			
+			writer.AddAttribute ("onload", "checkForTheme()");
 			writer.RenderBeginTag ("body");
 
-			writer.AddAttribute ("class", "navi");
+			writer.AddAttribute ("class", "container1");
+			writer.RenderBeginTag ("div");
+
+			writer.AddAttribute ("class", "header");
 			writer.RenderBeginTag ("div");
 
 			writer.AddAttribute ("id", "title");
 			writer.RenderBeginTag ("div");
-			writer.Write (description);
+			writer.Write (gallery_name);
 			writer.RenderEndTag ();
 
+			writer.AddAttribute ("class", "navi");
+			writer.RenderBeginTag ("div");
+
 			if (i > 0)
 				WritePageNav (writer, "prev", PhotoIndexPath (i - 1), Mono.Posix.Catalog.GetString("Prev"));
 
 			WritePageNav (writer, "index", IndexPath (i / perpage), Mono.Posix.Catalog.GetString("Index"));
 			
-			if (i < collection.Photos.Length -1)
+			if (i < collection.Count -1)
 				WritePageNav (writer, "next", PhotoIndexPath (i + 1), Mono.Posix.Catalog.GetString("Next"));
 
-			writer.RenderEndTag ();
+			writer.RenderEndTag (); //navi
+			
+			writer.RenderEndTag (); //header
 			
 			writer.AddAttribute ("class", "photo");
 			writer.RenderBeginTag ("div");
@@ -642,15 +686,51 @@ namespace FSpot {
 			writer.RenderBeginTag ("a");
 			
 			writer.AddAttribute ("src", PhotoWebPath (i));
+			writer.AddAttribute ("alt", "#");
 			writer.RenderBeginTag ("img");
 			writer.RenderEndTag ();
 			writer.RenderEndTag (); // a
 			
+			writer.AddAttribute ("id", "description");
 			writer.RenderBeginTag ("div");
-			writer.Write (collection.Photos [i].Description);
+			writer.Write (collection [i].Description);
 			writer.RenderEndTag ();
 
 			writer.RenderEndTag ();
+			
+		  //Style Selection Box
+			writer.AddAttribute ("id", "styleboxcontainer");
+			writer.RenderBeginTag ("div");
+			writer.AddAttribute ("id", "stylebox");
+			writer.AddAttribute ("style", "display: none;");
+			writer.RenderBeginTag ("div");
+			writer.RenderBeginTag("ul");
+			writer.RenderBeginTag("li");
+			writer.AddAttribute ("href", "#");
+			writer.AddAttribute ("title", "Dark");
+			writer.AddAttribute ("onclick", "setActiveStyleSheet('Dark')");
+			writer.RenderBeginTag("a");
+			writer.Write ("Dark");
+			writer.RenderEndTag (); //a
+			writer.RenderEndTag (); //li
+			writer.RenderBeginTag("li");
+			writer.AddAttribute ("href", "#");
+			writer.AddAttribute ("title", "Light");
+			writer.AddAttribute ("onclick", "setActiveStyleSheet('Light')");
+			writer.RenderBeginTag("a");
+			writer.Write ("Light");
+			writer.RenderEndTag (); //a
+			writer.RenderEndTag (); //li
+			writer.RenderEndTag (); //ul
+			writer.RenderEndTag (); //div stylebox
+			writer.RenderBeginTag ("div");
+			writer.Write ("<span class=\"style_toggle\">"); 
+			writer.Write ("<a href=\"javascript:toggle_stylebox()\">");
+			writer.Write ("<span id=\"showlink\">Show Styles</span><span id=\"hidelink\" ");
+			writer.Write ("style=\"display:none;\">Hide Styles</span></a></span>\n");
+			writer.RenderEndTag (); //div toggle
+			writer.RenderEndTag (); //div styleboxcontainer
+			writer.RenderEndTag (); //container1	
 
 			WriteFooter (writer);
 			
@@ -671,7 +751,7 @@ namespace FSpot {
 		
 		static string IndexTitle (int page)
 		{
-			return String.Format ("{0}", page);
+			return String.Format ("{0}", page + 1);
 		}
 
 		public void WriteHeader (System.Web.UI.HtmlTextWriter writer)
@@ -680,12 +760,19 @@ namespace FSpot {
 			writer.RenderBeginTag ("title");
 			writer.Write (gallery_name);
 			writer.RenderEndTag ();
-			
-			writer.AddAttribute ("type", "text/css");
-			writer.AddAttribute ("media", "screen");
-			writer.RenderBeginTag ("style");
-			writer.Write (String.Format ("@import url( {0} );", SubdirPath ("style", stylesheet)));
-			writer.RenderEndTag ();
+
+			writer.Write ("<link type=\"text/css\" rel=\"stylesheet\" href=\"");
+			writer.Write (String.Format ("{0}", "style/" + stylesheet));
+			writer.Write ("\" title=\"Dark\" media=\"screen\" />\n");
+
+			writer.Write ("<link type=\"text/css\" rel=\"prefetch ") ;
+			writer.Write ("alternate stylesheet\" href=\"");
+			writer.Write (String.Format ("{0}", "style/" + altstylesheet));
+			writer.Write ("\" title=\"Light\" media=\"screen\" />\n");
+
+			writer.Write ("<script src=\"script/" + javascript + "\"");
+			writer.Write (" type=\"text/javascript\"></script>\n");
+
 			writer.RenderEndTag ();
 		}
 		
@@ -712,22 +799,33 @@ namespace FSpot {
 			//writer.Indent = 4;
 
 			//writer.Write ("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
+			writer.WriteLine ("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">");
 			writer.RenderBeginTag ("html");
 			WriteHeader (writer);
 			
+			writer.AddAttribute ("onload", "checkForTheme()");
 			writer.RenderBeginTag ("body");
+			
 
-			writer.AddAttribute ("class", "navi");
+			
+			writer.AddAttribute ("class", "container1");
+			writer.RenderBeginTag ("div");
+
+			writer.AddAttribute ("class", "header");
 			writer.RenderBeginTag ("div");
 
 			writer.AddAttribute ("id", "title");
 			writer.RenderBeginTag ("div");
-			writer.Write (description);
-			writer.RenderEndTag ();
+			writer.Write (gallery_name);
+			writer.RenderEndTag (); //title div
 			
+			writer.AddAttribute ("class", "navi");
 			writer.RenderBeginTag ("div");
-			writer.Write (Mono.Posix.Catalog.GetString ("Pages:"));
-			writer.RenderEndTag ();
+
+			writer.AddAttribute ("class", "navilabel");
+			writer.RenderBeginTag ("div");
+			writer.Write (Mono.Posix.Catalog.GetString ("Page:"));
+			writer.RenderEndTag (); //pages div
 			
 			int i;
 			for (i = 0; i < PageCount; i++) {
@@ -737,28 +835,71 @@ namespace FSpot {
 				writer.AddAttribute ("href", IndexPath (i));
 				writer.RenderBeginTag ("a");
 				writer.Write (IndexTitle (i));
-				writer.RenderEndTag ();
+				writer.RenderEndTag (); //a
 				
-				writer.RenderEndTag ();
+				writer.RenderEndTag (); //navipage
 			}
-			writer.RenderEndTag ();
+			writer.RenderEndTag (); //navi
+			writer.RenderEndTag (); //header
 			
 			writer.AddAttribute ("class", "thumbs");
 			writer.RenderBeginTag ("div");
 			
 			int start = page_num * perpage;
-			int end = Math.Min (start + perpage, collection.Photos.Length);
+			int end = Math.Min (start + perpage, collection.Count);
 			for (i = start; i < end; i++) {
 				writer.AddAttribute ("href", PhotoIndexPath (i));
 				writer.RenderBeginTag ("a");
 				
 				writer.AddAttribute  ("src", PhotoThumbPath (i));
+				writer.AddAttribute  ("alt", "#");
 				writer.RenderBeginTag ("img");
 				writer.RenderEndTag ();
 				
-				writer.RenderEndTag ();
+				writer.RenderEndTag (); //a
 			}
 			
+			writer.RenderEndTag (); //thumbs
+			
+			writer.AddAttribute ("id", "gallery_description");
+			writer.RenderBeginTag ("div");
+			writer.Write (description);
+			writer.RenderEndTag (); //description
+			
+      //Style Selection Box
+			writer.AddAttribute ("id", "styleboxcontainer");
+			writer.RenderBeginTag ("div");
+			writer.AddAttribute ("id", "stylebox");
+			writer.AddAttribute ("style", "display: none;");
+			writer.RenderBeginTag ("div");
+			writer.RenderBeginTag("ul");
+			writer.RenderBeginTag("li");
+			writer.AddAttribute ("href", "#");
+			writer.AddAttribute ("title", "Dark");
+			writer.AddAttribute ("onclick", "setActiveStyleSheet('Dark')");
+			writer.RenderBeginTag("a");
+			writer.Write ("Dark");
+			writer.RenderEndTag (); //a
+			writer.RenderEndTag (); //li
+			writer.RenderBeginTag("li");
+			writer.AddAttribute ("href", "#");
+			writer.AddAttribute ("title", "Light");
+			writer.AddAttribute ("onclick", "setActiveStyleSheet('Light')");
+			writer.RenderBeginTag("a");
+			writer.Write ("Light");
+			writer.RenderEndTag (); //a
+			writer.RenderEndTag (); //li
+			writer.RenderEndTag (); //ul
+			writer.RenderEndTag (); //div stylebox
+			writer.RenderBeginTag ("div");
+			writer.Write ("<span class=\"style_toggle\">"); 
+			writer.Write ("<a href=\"javascript:toggle_stylebox()\">");
+			writer.Write ("<span id=\"showlink\">Show Styles</span><span id=\"hidelink\" ");
+			writer.Write ("style=\"display:none;\">Hide Styles</span></a></span>\n");
+			writer.RenderEndTag (); //div toggle
+			writer.RenderEndTag (); //div styleboxcontainer
+			writer.RenderEndTag (); //container1
+
 			WriteFooter (writer);
 			
 			writer.RenderEndTag (); //body
