@@ -7,26 +7,16 @@ using System;
 public class PhotoView : EventBox {
 	FSpot.Delay description_delay; 
 
-	public int CurrentPhoto {
-		get {
-			return photo_view.CurrentPhoto;
-		}
-		set {
-			photo_view.CurrentPhoto = value;
-		}
-	}
-
 	public FSpot.PhotoImageView View {
 		get {
 			return photo_view;
 		}
 	}
 
-	private bool CurrentPhotoValid () {
-		if (query == null || query.Photos.Length == 0 || CurrentPhoto >= Query.Photos.Length)
-			return false;
-
-		return true;
+	new public FSpot.BrowsablePointer Item {
+		get {
+			return photo_view.Item;
+		}
 	}
 
 	private PhotoStore photo_store;
@@ -89,12 +79,29 @@ public class PhotoView : EventBox {
 	private struct SelectionConstraint {
 		public string Label;
 		public double XyRatio;
+		
+		public SelectionConstraint (string label, double ratio)
+		{
+			Label = label;
+			XyRatio = ratio;
+		}
 	}
 
 	private OptionMenu constraints_option_menu;
 	private int selection_constraint_ratio_idx;
 
-	private static SelectionConstraint [] constraints;
+	private static SelectionConstraint [] constraints = {
+		new SelectionConstraint (Mono.Posix.Catalog.GetString ("No Constraint"), 0.0),
+		new SelectionConstraint (Mono.Posix.Catalog.GetString ("4 x 3 (Book)"), 4.0 / 3.0),
+		new SelectionConstraint (Mono.Posix.Catalog.GetString ("4 x 6 (Postcard)"), 6.0 / 4.0),
+		new SelectionConstraint (Mono.Posix.Catalog.GetString ("5 x 7 (L, 2L)"), 7.0 / 5.0),
+		new SelectionConstraint (Mono.Posix.Catalog.GetString ("8 x 10"), 10.0 / 8.0),
+		new SelectionConstraint (Mono.Posix.Catalog.GetString ("4 x 3 Portrait (Book)"), 3.0 / 4.0),
+		new SelectionConstraint (Mono.Posix.Catalog.GetString ("4 x 6 Portrait (Postcard)"), 4.0 / 6.0),
+		new SelectionConstraint (Mono.Posix.Catalog.GetString ("5 x 7 Portrait (L, 2L)"), 5.0 / 7.0),
+		new SelectionConstraint (Mono.Posix.Catalog.GetString ("8 x 10 Portrait"), 8.0 / 10.0),
+		new SelectionConstraint (Mono.Posix.Catalog.GetString ("Square"), 1.0)
+	};
 
 	private void HandleSelectionConstraintOptionMenuActivated (object sender, EventArgs args)
 	{
@@ -128,9 +135,9 @@ public class PhotoView : EventBox {
 
 	private void UpdateButtonSensitivity ()
 	{
-		bool valid = CurrentPhotoValid ();
-		bool prev = valid && CurrentPhoto > 0;
-		bool next = valid && CurrentPhoto < query.Photos.Length -1;
+		bool valid = photo_view.Item.IsValid;
+		bool prev = valid && Item.Index > 0;
+		bool next = valid && Item.Index < query.Count - 1;
 
 		display_previous_button.Sensitive = prev;
 		display_next_button.Sensitive = next;
@@ -148,19 +155,19 @@ public class PhotoView : EventBox {
 			if (Query.Photos.Length == 0)
 				count_label.Text = String.Format ("{0} of {1}", 0, 0);
 			else 
-				count_label.Text = String.Format ("{0} of {1}", CurrentPhoto + 1, Query.Photos.Length);
+				count_label.Text = String.Format ("{0} of {1}", Item.Index + 1, Query.Count);
 		}
 	}
 
 	private void UpdateDescriptionEntry ()
 	{
 		description_entry.Changed -= HandleDescriptionChanged;
-		if (Query.Photos.Length > 0 && CurrentPhoto < Query.Photos.Length) {
+		if (Item.IsValid) {
 			if (description_entry.Sensitive == false)
 				description_entry.Sensitive = true;
-
-			if (description_entry.Text != Query.Photos[CurrentPhoto].Description) {
-				description_entry.Text = Query.Photos[CurrentPhoto].Description;
+			
+			if (description_entry.Text != Item.Current.Description) {
+				description_entry.Text = Item.Current.Description;
 			}
 		} else {
 			description_entry.Sensitive = false;
@@ -182,20 +189,6 @@ public class PhotoView : EventBox {
 			UpdateFinished (this);
 	}
 
-
-	// Browsing.
-
-	private void DisplayNext ()
-	{
-		photo_view.Next ();
-	}
-
-	private void DisplayPrevious ()
-	{
-		photo_view.Prev ();
-	}
-
-
 	// Event handlers.
 	private void HandleButtonPressEvent (object sender, ButtonPressEventArgs args)
 	{
@@ -208,12 +201,12 @@ public class PhotoView : EventBox {
 
 	private void HandleDisplayNextButtonClicked (object sender, EventArgs args)
 	{
-		DisplayNext ();
+		View.Item.MoveNext ();
 	}
 
 	private void HandleDisplayPreviousButtonClicked (object sender, EventArgs args)
 	{
-		DisplayPrevious ();
+		View.Item.MovePrevious ();
 	}
 
 
@@ -240,7 +233,7 @@ public class PhotoView : EventBox {
 			return;
 		}
 
-		Photo photo = query.Photos [CurrentPhoto];
+		Photo photo = (Photo)Item.Current;
 		Exif.ExifData exif_data = new Exif.ExifData (photo.DefaultVersionPath);
 
 		Pixbuf edited;
@@ -272,13 +265,13 @@ public class PhotoView : EventBox {
 				PixbufUtils.SaveJpeg (edited, photo.DefaultVersionPath, 
 						      95, exif_data);
 				FSpot.ThumbnailGenerator.Create (photo.DefaultVersionPath).Dispose ();
-				query.Commit (CurrentPhoto);
+				query.Commit (Item.Index);
 			} else {
 				// FIXME we need to invalidate the thumbnail in the cache as well
 				PixbufUtils.SaveJpeg (edited, photo.DefaultVersionPath, 
 						      95, exif_data);
 				FSpot.ThumbnailGenerator.Create (photo.DefaultVersionPath).Dispose ();
-				query.MarkChanged (CurrentPhoto);
+				query.MarkChanged (Item.Index);
 			}
 		} catch (System.Exception e) {
 			string msg = Mono.Posix.Catalog.GetString ("Error editing photo");
@@ -311,24 +304,37 @@ public class PhotoView : EventBox {
 	{
 		if (description_delay.IsPending) {
 			description_delay.Stop ();
+
+#if true // UPDATE_EXIF_DESCRIPTION
+			Photo photo = query.Photos [description_photo];
+			FSpot.ImageFile img = FSpot.ImageFile.Create (photo.DefaultVersionPath);
+			if (img is FSpot.JpegFile) {
+				FSpot.JpegFile jimg = img as FSpot.JpegFile;
+				jimg.Description = photo.Description;
+				jimg.SaveMetaData (photo.DefaultVersionPath);
+			}
+			//Query.Store.Commit (photo);
 			Query.Commit (description_photo);
+#else
+			Query.Commit (description_photo);
+#endif
 		}
 		return true;
 	}
 
 	private void HandleDescriptionChanged (object sender, EventArgs args) {
-		if (!CurrentPhotoValid ())
+		if (!Item.IsValid)
 			return;
 
-		Query.Photos[CurrentPhoto].Description = description_entry.Text;
+		((Photo)Item.Current).Description = description_entry.Text;
 
 		if (description_delay.IsPending)
-			if (description_photo == CurrentPhoto)
+			if (description_photo == Item.Index)
 				description_delay.Stop ();
 			else
 				CommitPendingChanges ();
 
-		description_photo = CurrentPhoto;
+		description_photo = Item.Index;
 		description_delay.Start ();
 	}
 
@@ -345,49 +351,12 @@ public class PhotoView : EventBox {
 		}
 	}
 
-	static PhotoView ()
-	{
-		constraints = new SelectionConstraint [10];
-
-		constraints[0].Label = "No Constraint";
-		constraints[0].XyRatio = 0.0;
-
-		constraints[1].Label = "4 x 3 (Book)";
-		constraints[1].XyRatio = 4.0 / 3.0;
-
-		constraints[2].Label = "4 x 6 (Postcard)";
-		constraints[2].XyRatio = 6.0 / 4.0;
-
-		constraints[3].Label = "5 x 7 (L, 2L)";
-		constraints[3].XyRatio = 7.0 / 5.0;
-
-		constraints[4].Label = "8 x 10";
-		constraints[4].XyRatio = 10.0 / 8.0;
-
-		constraints[5].Label = "4 x 3 Portrait (Book)";
-		constraints[5].XyRatio = 3.0 / 4.0;
-
-		constraints[6].Label = "4 x 6 Portrait (Postcard)";
-		constraints[6].XyRatio = 4.0 / 6.0;
-
-		constraints[7].Label = "5 x 7 Portrait (L, 2L)";
-		constraints[7].XyRatio = 5.0 / 7.0;
-
-		constraints[8].Label = "8 x 10 Portrait";
-		constraints[8].XyRatio = 8.0 / 10.0;
-
-		constraints[9].Label = "Square";
-		constraints[9].XyRatio = 1.0;
-	}
-
 	private void HandlePhotoChanged (FSpot.PhotoImageView view)
 	{
 		CommitPendingChanges ();
-		bool valid = CurrentPhotoValid ();
-
 		Update ();
 
-		tag_view.Current = valid ? query.Photos [CurrentPhoto] : null;
+		tag_view.Current = (Photo)(Item.Current);
 
 		if (this.PhotoChanged != null)
 			PhotoChanged (this);
@@ -398,13 +367,15 @@ public class PhotoView : EventBox {
 		CommitPendingChanges ();
 	}
 
+	Gtk.Tooltips tips = new Gtk.Tooltips ();
+
 	public PhotoView (FSpot.PhotoQuery query, PhotoStore photo_store)
 		: base ()
 	{
 		this.query = query;
 		this.photo_store = photo_store;
 
-		description_delay = new FSpot.Delay (500, new GLib.IdleHandler (CommitPendingChanges));
+		description_delay = new FSpot.Delay (1000, new GLib.IdleHandler (CommitPendingChanges));
 		this.Destroyed += HandleDestroy;
 
 
@@ -491,9 +462,15 @@ public class PhotoView : EventBox {
 		display_next_button.Clicked += new EventHandler (HandleDisplayNextButtonClicked);
 		toolbar_hbox.PackStart (display_next_button, false, true, 0);
 
+		tips.Enable ();
+
+
 		UpdateButtonSensitivity ();
 
 		vbox.ShowAll ();
+		tips.SetTip (crop_button, Mono.Posix.Catalog.GetString ("Crop image to selected area"), "so is this one");
+		tips.SetTip (redeye_button, Mono.Posix.Catalog.GetString ("Remove redeye from selected area"), "this is one too");
+		tips.SetTip (color_button, Mono.Posix.Catalog.GetString ("Adjust the image colors"), "This is a tip");
 	}
 
 }
