@@ -4,6 +4,16 @@ using System.Runtime.InteropServices;
 using System;
 using System.IO;
 
+/**
+  1        2       3      4         5            6           7          8
+
+888888  888888      88  88      8888888888  88                  88  8888888888
+88          88      88  88      88  88      88  88          88  88      88  88
+8888      8888    8888  8888    88          8888888888  8888888888          88
+88          88      88  88
+88          88  888888  888888
+**/
+
 public enum PixbufOrientation {
 	TopLeft = 1,
 	TopRight = 2,
@@ -16,6 +26,31 @@ public enum PixbufOrientation {
 }
 
 class PixbufUtils {
+		
+	static public PixbufOrientation Rotate270 (PixbufOrientation orientation)
+	{
+		PixbufOrientation [] rot = new PixbufOrientation [] {
+			PixbufOrientation.LeftBottom, 
+			PixbufOrientation.LeftTop,
+			PixbufOrientation.RightTop,
+			PixbufOrientation.RightBottom, 
+			PixbufOrientation.BottomLeft,
+			PixbufOrientation.TopLeft,
+			PixbufOrientation.TopRight,
+			PixbufOrientation.BottomRight
+		};
+
+		orientation = rot [((int)orientation) -1];
+		return orientation;
+	}
+
+	static public PixbufOrientation Rotate90 (PixbufOrientation orientation)
+	{
+		orientation = Rotate270 (orientation);
+		orientation = Rotate270 (orientation);
+		orientation = Rotate270 (orientation);
+		return orientation;
+	}
 
 	public static Pixbuf ErrorPixbuf = PixbufUtils.LoadFromAssembly ("f-spot-question-mark.png");
 	public static Pixbuf LoadingPixbuf = PixbufUtils.LoadFromAssembly ("f-spot-loading.png");
@@ -25,26 +60,38 @@ class PixbufUtils {
 		return Math.Max (pixbuf.Width, pixbuf.Height);
 	}
 
-	public static void Fit (Pixbuf pixbuf,
-				int dest_width, int dest_height,
-				bool upscale_smaller,
-				out int fit_width, out int fit_height)
+	public static double Fit (Pixbuf pixbuf,
+				  int dest_width, int dest_height,
+				  bool upscale_smaller,
+				  out int fit_width, out int fit_height)
 	{
-		if (pixbuf.Width == 0 || pixbuf.Height == 0) {
+		return Fit (pixbuf.Width, pixbuf.Height, 
+			    dest_width, dest_height, 
+			    upscale_smaller, 
+			    out fit_width, out fit_height);
+	}
+
+	public static double Fit (int orig_width, int orig_height,
+				  int dest_width, int dest_height,
+				  bool upscale_smaller,
+				  out int fit_width, out int fit_height)
+	{
+		if (orig_width == 0 || orig_height == 0) {
 			fit_width = 0;
 			fit_height = 0;
-			return;
+			return 0.0;
 		}
 
-		double scale = Math.Min (dest_width / (double)pixbuf.Width,
-					 dest_height / (double)pixbuf.Height);
+		double scale = Math.Min (dest_width / (double)orig_width,
+					 dest_height / (double)orig_height);
 		
 		if (scale > 1.0 && !upscale_smaller)
 			scale = 1.0;
 
-		fit_width = (int)(scale * pixbuf.Width);
-		fit_height = (int)(scale * pixbuf.Height);
-
+		fit_width = (int)(scale * orig_width);
+		fit_height = (int)(scale * orig_height);
+		
+		return scale;
 	}
 
 
@@ -80,16 +127,18 @@ class PixbufUtils {
 
 			double scale = Math.Min (max_width / (double)args.Width,
 						 max_height / (double)args.Height);
+
 			
 			int scale_width = (int)(scale * args.Width);
 			int scale_height = (int)(scale * args.Height);
 
-			SetSize (scale_width, scale_height);
+			if (scale < 1.0)
+				SetSize (scale_width, scale_height);
 		}
 
 		public Pixbuf LoadFromFile (string path)
 		{
-			FileStream fs;
+			FileStream fs = null;
 
 			try {
 				orientation = GetOrientation (path);
@@ -97,7 +146,7 @@ class PixbufUtils {
 				int count;
 				
 				byte [] data = new byte [8192];
-				while (((count = fs.Read (data, 0, 8192)) > 0) && this.Write (data, (uint)count))
+				while (((count = fs.Read (data, 0, 8192)) > 0) && this.Write (data, (ulong)count))
 					;
 				
 				this.Close ();
@@ -109,40 +158,68 @@ class PixbufUtils {
 				}
 				return rotated;
 			} catch (Exception e) {
-				System.Console.Write ("Error loading image {0}", path);
+				System.Console.WriteLine ("Error loading image {0}", path);
 				return null;
+			} finally {
+				if (fs != null)
+					fs.Close ();
 			}
 
 		}
 	}
 
+	public static Pixbuf ShallowCopy (Pixbuf pixbuf)
+	{
+		Pixbuf result = new Pixbuf (pixbuf, 0, 0, pixbuf.Width, pixbuf.Height);
+		CopyThumbnailOptions (pixbuf, result);
+		return result;
+	}
+
 	public static Pixbuf ScaleToMaxSize (Pixbuf pixbuf, int width, int height)
+	{
+		return ScaleToMaxSize (pixbuf, width, height, true);
+	}	
+
+	public static Pixbuf ScaleToMaxSize (Pixbuf pixbuf, int width, int height, bool upscale)
 	{
 		double scale = Math.Min  (width / (double)pixbuf.Width, height / (double)pixbuf.Height);
 		int scale_width = (int)(scale * pixbuf.Width);
 		int scale_height = (int)(scale * pixbuf.Height);
 
-		Gdk.Pixbuf result = pixbuf.ScaleSimple (scale_width, scale_height, Gdk.InterpType.Bilinear);
+		Gdk.Pixbuf result;
+		if (upscale || (scale < 1.0))
+			result = pixbuf.ScaleSimple (scale_width, scale_height, (scale_width > 20) ? Gdk.InterpType.Bilinear : Gdk.InterpType.Nearest);
+		else
+			result = pixbuf.Copy ();
 
 		CopyThumbnailOptions (pixbuf, result);
 
-		return pixbuf.ScaleSimple (scale_width, scale_height, Gdk.InterpType.Bilinear);
+		return result;
 	}
 		
 	static public Pixbuf LoadAtMaxSize (string path, int max_width, int max_height)
 	{
+#if true
 		PixbufUtils.AspectLoader loader = new AspectLoader (max_width, max_height);
 		return loader.LoadFromFile (path);
+#else
+		int width, height;
+		JpegUtils.GetSize (path, out width, out height);
+		PixbufUtils.Fit (width, height, max_width, max_height, false, out width, out height);
+		Gdk.Pixbuf image = JpegUtils.LoadScaled (path, width, height);
+		
+		return image;
+#endif
 	}
 
-	static private Pixbuf LoadFromStream (System.IO.Stream input)
+	static public Pixbuf LoadFromStream (System.IO.Stream input)
 	{
 		Gdk.PixbufLoader loader = new Gdk.PixbufLoader ();
 		byte [] buffer = new byte [8192];
 		int n;
 
 		while ((n = input.Read (buffer, 0, 8192)) != 0)
-			loader.Write (buffer, (uint) n);
+			loader.Write (buffer, (ulong) n);
 		
 		loader.Close ();
 		return loader.Pixbuf;
@@ -173,20 +250,43 @@ class PixbufUtils {
 		}
 	}
 
+	public static void Save (Gdk.Pixbuf pixbuf, System.IO.Stream stream, string type, string [] options, string [] values)
+	{
+		byte [] data;
+
+		data = PixbufUtils.Save (pixbuf, type, options, values);
+		stream.Write (data, 0, data.Length);
+	}
+
 	[DllImport("libgdk_pixbuf-2.0-0.dll")]
 	static extern bool gdk_pixbuf_save_to_bufferv (IntPtr raw, out IntPtr data, out uint length, string type, 
 						       string [] keys, string [] values, out IntPtr error);
+
 					
-	public byte [] Save (Gdk.Pixbuf pixbuf, string type)
+	public static byte [] Save (Gdk.Pixbuf pixbuf, string type, string [] options, string [] values)
 	{
 		IntPtr error = IntPtr.Zero;
 		IntPtr data;
 		uint length;
+		string [] terminated_options = null;
+		string [] terminated_values = null;
+
+		if (options != null) {
+			terminated_options = new string [options.Length + 1];
+			Array.Copy (options, terminated_options, options.Length);
+		}
+
+		if (options != null) {
+			terminated_values = new string [values.Length + 1];
+			Array.Copy (values, terminated_values, values.Length);
+		}
+
 		gdk_pixbuf_save_to_bufferv (pixbuf.Handle, 
 					    out data, 
 					    out length, 
 					    type,
-					    null, null,
+					    terminated_options,
+					    terminated_values,
 					    out error);
 
 		if (error != IntPtr.Zero) 
@@ -221,6 +321,23 @@ class PixbufUtils {
 		return icon;
 	}
 		
+	static public Pixbuf LoadFromScreen (Gdk.Window win) {
+		Gdk.Screen screen = win.Screen;
+		Drawable d = screen.RootWindow;
+		int monitor = screen.GetMonitorAtWindow (win);
+		Gdk.Rectangle geom = screen.GetMonitorGeometry (monitor);
+		
+		//
+		// We use the screen width and height because that reflects
+		// the current resolution, the RootWindow can actually be different.
+		//
+
+		Pixbuf buf = new Pixbuf (Colorspace.Rgb, false, 8, geom.Width, geom.Height);
+		
+		return buf.GetFromDrawable (d,
+					    d.Colormap, geom.X, geom.Y, 0, 0, 
+					    geom.Width, geom.Height);
+	}
 
 	static public Pixbuf LoadFromScreen () {
 		Screen screen = Display.Default.GetScreen (0);
@@ -249,6 +366,132 @@ class PixbufUtils {
 		}
 	}
 
+	[DllImport ("libc")]
+	static extern int rename (string oldpath, string newpath);
+
+	public static void SaveAtomic (Gdk.Pixbuf src, string filename, string type, string [] keys, string [] values)
+	{
+			string tmpname = filename + ".tmp";
+			src.Savev (tmpname, type, keys, values);
+			if (rename (tmpname, filename) < 0)
+				throw new Exception ("Error renaming file");
+	}
+
+	public static Gdk.Pixbuf ScaleToAspect (Gdk.Pixbuf orig, int width, int height)
+	{
+		Gdk.Rectangle pos;
+		double scale = Fit (orig, width, height, false, out pos.Width, out pos.Height);
+		pos.X = (width - pos.Width) / 2;
+		pos.Y = (height - pos.Height) / 2;
+
+		Pixbuf scaled = new Pixbuf (Colorspace.Rgb, false, 8, width, height);
+		scaled.Fill (0x000000); 
+
+		//System.Console.WriteLine ("pos= {0}", pos.ToString ());
+
+		orig.Composite (scaled, pos.X, pos.Y, 
+				pos.Width, pos.Height,
+				pos.X, pos.Y, scale, scale,
+				Gdk.InterpType.Bilinear,
+				255);
+
+		return scaled;
+	}
+
+	public static string Resize (string orig_path, int size, bool copy_meta)
+	{
+		string version_path = System.IO.Path.GetTempFileName ();
+		Resize (orig_path, version_path, size, copy_meta);
+		return version_path;
+	}
+
+	public static void Resize (string orig_path, string dest_path, int size, bool copy_meta)
+	{
+		Exif.ExifData exif_data;
+		if (copy_meta)
+			exif_data = new Exif.ExifData (orig_path);
+		else 
+			exif_data = new Exif.ExifData ();
+
+		Gdk.Pixbuf image = PixbufUtils.LoadAtMaxSize (orig_path, size, size);
+		PixbufUtils.SaveJpeg (image, dest_path, 95, exif_data);
+	}
+	
+
+	[StructLayout(LayoutKind.Sequential)]
+	public unsafe struct FPixbufJpegMarker {
+		public int type;
+		public byte *data;
+		public int length;
+	}
+
+	[DllImport ("libfspot")]
+	static extern bool f_pixbuf_save_jpeg (IntPtr src, string path, int quality, FPixbufJpegMarker [] markers, int num_markers);
+
+	public static void SaveJpeg (Pixbuf pixbuf, string path, int quality, Exif.ExifData exif_data)
+	{
+		// The DCF spec says thumbnails should be 160x120 always
+		Pixbuf thumbnail = ScaleToAspect (pixbuf, 160, 120);
+		byte [] thumb_data = Save (thumbnail, "jpeg", null, null);
+		exif_data.Data = thumb_data;
+		thumbnail.Dispose ();
+
+		// Most of the things we will set will be in the 0th ifd
+		Exif.ExifContent content = exif_data.GetContents (Exif.Ifd.Zero);
+
+		// reset the orientation tag the default is top/left
+		content.GetEntry (Exif.Tag.Orientation).Reset ();
+
+		// set the write time in the datetime tag
+		content.GetEntry (Exif.Tag.DateTime).Reset ();
+
+		// set the software tag
+		content.GetEntry (Exif.Tag.Software).SetData (FSpot.Defines.PACKAGE + " version " + FSpot.Defines.VERSION);
+
+		byte [] data = exif_data.Save ();
+		FPixbufJpegMarker [] marker = new FPixbufJpegMarker [0];
+		bool result = false;
+
+		unsafe {
+			if (data.Length > 0) {
+				
+				fixed (byte *p = data) {
+					marker = new FPixbufJpegMarker [1];
+					marker [0].type = 0xe1; // APP1 marker
+					marker [0].data = p;
+					marker [0].length = data.Length;
+					
+					result = f_pixbuf_save_jpeg (pixbuf.Handle, path, quality, marker, marker.Length);
+				}					
+			} else
+				result = f_pixbuf_save_jpeg (pixbuf.Handle, path, quality, marker, marker.Length);
+			
+		}
+
+		if (result == false)
+			throw new System.Exception ("Error Saving File");
+	}
+
+	[DllImport ("libgtk-win32-2.0-0.dll")]
+	extern static IntPtr gtk_icon_theme_get_default ();
+
+	[DllImport ("libgtk-win32-2.0-0.dll")]
+	extern static IntPtr gtk_icon_theme_load_icon (IntPtr theme, string name, int size, int flags, IntPtr error);
+	
+	public static Gdk.Pixbuf LoadThemeIcon (string name, int size)
+	{
+		try {
+			IntPtr native = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), name, size, 0, IntPtr.Zero);
+			if (native != IntPtr.Zero) {
+				Gdk.Pixbuf ret = (Gdk.Pixbuf) GLib.Object.GetObject(native, true);
+				return ret;
+			}
+		} catch (System.Exception e) {
+			System.Console.Write (e.ToString ());
+		}
+		return null;
+	}
+
 
 	[DllImport ("libfspot")]
 	static extern IntPtr f_pixbuf_unsharp_mask (IntPtr src, double radius, double amount, double threshold);
@@ -260,6 +503,20 @@ class PixbufUtils {
 		return ret;
 	}	
 	
+	[DllImport ("libfspot")]
+	static extern void f_pixbuf_remove_redeye (IntPtr src);
+	
+	public static Gdk.Pixbuf RemoveRedeye (Gdk.Pixbuf src, Gdk.Rectangle area)
+	{
+		Gdk.Pixbuf copy = src.Copy ();
+		Gdk.Pixbuf selection = new Gdk.Pixbuf (copy, area.X, area.Y, area.Width, area.Height);
+
+		f_pixbuf_remove_redeye (selection.Handle);
+		selection.Dispose ();
+
+		return copy;
+	}
+
 	public static unsafe Pixbuf ColorAdjust (Pixbuf src, double brightness, double contrast,
 					  double hue, double saturation, int src_color, int dest_color)
 	{
@@ -315,20 +572,44 @@ class PixbufUtils {
 		
 	}
 
-	public static PixbufOrientation GetOrientation (string path)
+	public static Gdk.Pixbuf GetThumbnail (Exif.ExifData data)
 	{
-		ExifData exif = new ExifData (path);
-		byte [] value = exif.LookupData (ExifTag.Orientation);
-		PixbufOrientation orientation = PixbufOrientation.TopLeft;
+		byte [] thumb_data = data.Data;
+		if (thumb_data.Length > 0) {
+			PixbufOrientation orientation = GetOrientation (data);
+			MemoryStream mem = new MemoryStream (thumb_data);
+			Gdk.Pixbuf thumb = new Gdk.Pixbuf (mem);
+			Gdk.Pixbuf rotated = PixbufUtils.TransformOrientation (thumb, orientation);
+			
+			if (rotated != thumb)
+				thumb.Dispose ();
+			
+			return rotated;
+		}
+		return null;
+	}
 
-		if (value != null) {
-			orientation = (PixbufOrientation)value [0];
+	public static PixbufOrientation GetOrientation (Exif.ExifData data)
+	{
+		PixbufOrientation orientation = PixbufOrientation.TopLeft;
+		
+		Exif.ExifEntry e = data.GetContents (Exif.Ifd.Zero).Lookup (Exif.Tag.Orientation);
+
+		if (e != null) {
+			ushort [] value = e.GetDataUShort ();
+			orientation = (PixbufOrientation) value [0];
 		}
 
 		return orientation;
 	}
+	
+	public static PixbufOrientation GetOrientation (string path)
+	{
+		FSpot.ImageFile img = FSpot.ImageFile.Create (path);
+		return img.Orientation;
+	}
 
-	[DllImport("gnomeui-2")]
+	[DllImport("libgnomeui-2-0.dll")]
 	static extern IntPtr gnome_thumbnail_scale_down_pixbuf(IntPtr pixbuf, int dest_width, int dest_height);
 
 	public static Gdk.Pixbuf ScaleDown (Gdk.Pixbuf src, int width, int height)
@@ -345,7 +626,9 @@ class PixbufUtils {
 	public static Gdk.Pixbuf TransformOrientation (Gdk.Pixbuf src, PixbufOrientation orientation, bool copy_data)
 	{
 		Gdk.Pixbuf pixbuf;
-
+		if (src == null)
+			return null;
+		
 		switch (orientation) {
 		case PixbufOrientation.LeftTop:
 		case PixbufOrientation.LeftBottom:
@@ -399,13 +682,14 @@ class PixbufUtils {
 			area.Y = total_height - args.Y - args.Height;
 			break;
 		case PixbufOrientation.LeftTop:
-			area.X = area.Y;
-			area.Y = area.X;
+			area.X = args.Y;
+			area.Y = args.X;
 			area.Width = args.Height;
 			area.Height = args.Width;
 			break;
 		case PixbufOrientation.RightBottom:
 			area.X = total_height - args.Y - args.Height;
+			area.Y = total_width - args.X - args.Width;
 			area.Width = args.Height;
 			area.Height = args.Width;
 			break;
@@ -432,15 +716,16 @@ class PixbufUtils {
 	{
 		Gdk.Rectangle area = TransformOrientation (src, args, orientation);
 
-		int step = 512;
+		int step = 256;
 
 		Gdk.Rectangle rect = new Gdk.Rectangle (args.X, args.Y, 
 							Math.Min (step, args.Width),
 							Math.Min (step, args.Height));
 
+		Gdk.Rectangle trect = TransformOrientation (src, rect, orientation);
 		Gdk.Pixbuf tmp = new Gdk.Pixbuf (src.Colorspace, src.HasAlpha, 
 						 src.BitsPerSample,
-						 rect.Height, rect.Width);
+						 trect.Width, trect.Height);
 
 		Gdk.Rectangle subarea;
 		while (rect.Y < args.Y + args.Height) {
@@ -451,14 +736,11 @@ class PixbufUtils {
 			        Gdk.Pixbuf ssub = new Gdk.Pixbuf (src, subarea.X, subarea.Y,
 								  subarea.Width, subarea.Height);
 
-				
 				Gdk.Pixbuf tsub = new Gdk.Pixbuf (tmp, 0, 0, trans.Width, trans.Height);
 
 				CopyWithOrientation (ssub, tsub, orientation);
-				
-				tsub.CopyArea (0, 0, trans.Width, trans.Height, dest, trans.X, trans.Y);
-				
 
+				tsub.CopyArea (0, 0, trans.Width, trans.Height, dest, trans.X, trans.Y);
 				
 				ssub.Dispose ();
 				tsub.Dispose ();
@@ -471,7 +753,6 @@ class PixbufUtils {
 		tmp.Dispose ();
 		return area;
 	}
-
 	// Bindings from libf.
 
 	[DllImport ("libfspot")]
