@@ -13,6 +13,8 @@ public class Driver {
 		Gnome.Vfs.Vfs.Initialize ();
 		StockIcons.Initialize ();
 
+		Mono.Posix.Catalog.Init ("f-spot", FSpot.Defines.LOCALE_DIR);
+
 		// FIXME: Error checking is non-existant here...
 
 		string base_directory = FSpot.Global.BaseDirectory;
@@ -20,27 +22,45 @@ public class Driver {
 			Directory.CreateDirectory (base_directory);
 
 		Db db = new Db (Path.Combine (base_directory, "photos.db"), true);
-		
+
 		Gtk.Window.DefaultIconList = new Gdk.Pixbuf [] {PixbufUtils.LoadFromAssembly ("f-spot-logo.png")};
 
-		new MainWindow (db);
-		
-		ParseCommands (args);
+		bool view_only = false;
+		bool import = db.Empty;
+		string import_path = null;
 
-		program.Run ();
-	}
-
-	private static void  ParseCommands (string [] args) 
-	{
 		for (int i = 0; i < args.Length; i++) {
-			if (args [i] == "--import") {
-				if (++i < args.Length && (File.Exists (args [i]) || Directory.Exists (args[i]))) {
-					UriList urilist = new UriList (new string [] {args [i]});
-					MainWindow.Toplevel.ImportUriList (urilist);
+			switch (args [i]) {
+			case "--import":
+				if (++i < args.Length && (args [i].StartsWith ("gphoto2:") || File.Exists (args [i]) || Directory.Exists (args[i]))) {
+					import_path = args [i];
 				} else {
 					System.Console.WriteLine ("no valid path to import from");
 				}
+				import = true;
+				break;
+			case "--view":
+				if (++i < args.Length && (File.Exists (args [i]) || Directory.Exists (args[i]))) {
+					new FSpot.SingleView (args [i]);
+					view_only = true;
+				} else {
+					System.Console.WriteLine ("no valid path to import from");
+				}
+				break;
+			}
+
+		}
+
+		if (!view_only) {
+			new MainWindow (db);
+			if (import) {
+				if (import_path.StartsWith ("gphoto2:"))
+					MainWindow.Toplevel.ImportCamera (import_path);
+				else
+					MainWindow.Toplevel.ImportFile (import_path);
 			}
 		}
+
+		program.Run ();
 	}
 }
